@@ -1022,6 +1022,70 @@ static void add_bond_info(struct netcf *ncf,
 }
 
 
+static void add_link_info(struct netcf *ncf,
+                          const char *ifname, int ifindex ATTRIBUTE_UNUSED,
+                          xmlDocPtr doc, xmlNodePtr root) {
+    char errbuf[128];
+    xmlNodePtr link_node = NULL;
+    xmlAttrPtr prop = NULL;
+    char *path = NULL;
+    size_t length;
+    char *state = NULL;
+    char *speed = NULL;
+    char *nl;
+
+    link_node = xml_node(doc, root, "link");
+    ERR_NOMEM(link_node == NULL, ncf);
+
+    xasprintf(&path, "/sys/class/net/%s/operstate", ifname);
+    ERR_NOMEM(!path, ncf);
+    state = read_file(path, &length);
+    FREE(path);
+    ERR_THROW_STRERROR(!state, ncf, EFILE, "Failed to read %s", path);
+    if ((nl = strchr(state, '\n')))
+        *nl = 0;
+    prop = xmlSetProp(link_node, BAD_CAST "state", BAD_CAST state);
+    ERR_NOMEM(!prop, ncf);
+
+    if (!strcmp(state, "up")) {
+        xasprintf(&path, "/sys/class/net/%s/speed", ifname);
+        ERR_NOMEM(path == NULL, ncf);
+        speed = read_file(path, &length);
+        if (!speed && errno == EINVAL) {
+            /* attempts to read $ifname/speed result in EINVAL if the
+             * interface is ifconfiged down (which isn't exactly the
+             * same as an operstate of "down").
+             */
+            speed = strdup("0");
+            ERR_NOMEM(!speed, ncf);
+        }
+        ERR_THROW_STRERROR(!speed, ncf, EFILE, "Failed to read %s", path);
+        if ((nl = strchr(speed, '\n')))
+            *nl = 0;
+    } else {
+        /* When the link state is "down" (and most/all other states
+         * except "up"), different drivers report a different value
+         * for speed. In one local sample, the following were seen:
+         * "10", "65535", "4294967295". Since the effective speed is
+         * "0", just change it to that whenever the link state is not
+         * "up".
+         */
+        FREE(speed);
+        speed = strdup("0");
+        ERR_NOMEM(!speed, ncf);
+    }
+
+    prop = xmlSetProp(link_node, BAD_CAST "speed", BAD_CAST speed);
+    ERR_NOMEM(prop == NULL, ncf);
+
+ error:
+    FREE(path);
+    FREE(state);
+    FREE(speed);
+    return;
+}
+
+
 static void add_type_specific_info(struct netcf *ncf,
                                    const char *ifname, int ifindex,
                                    xmlDocPtr doc, xmlNodePtr root) {
@@ -1041,6 +1105,11 @@ static void add_type_specific_info(struct netcf *ncf,
         ERR_NOMEM(prop == NULL, ncf);
     }
 
+    if (iftype != NETCF_IFACE_TYPE_BRIDGE) {
+        add_link_info(ncf, ifname, ifindex, doc, root);
+        ERR_BAIL(ncf);
+    }
+
     switch (iftype) {
         case NETCF_IFACE_TYPE_ETHERNET:
             add_ethernet_info(ncf, ifname, ifindex, doc, root);
